// zoom screen saver 
// by douglas irving repetto
// based on code from be inc.
//
// randomly zooms into some region of the screen.

#include <add-ons/screen_saver/ScreenSaver.h> 
#include <interface/StringView.h> 
#include <BeBuild.h>
#include <Bitmap.h> 
#include <stdlib.h>
#include <stdio.h>

extern "C" { 
_EXPORT BScreenSaver *instantiate_screen_saver( BMessage *msg, image_id id ); 
} 

class MinimalSaver : public BScreenSaver 
{ 
public: 
    MinimalSaver( BMessage *archive, image_id image ); 
    virtual ~MinimalSaver( void ); 
    
    virtual void StartConfig( BView *view ); 
    virtual status_t StartSaver( BView *view, bool preview ); 
    virtual void Draw( BView *view, int32 frame ); 
    
	bool		doingPreview;
	BRect screenFrame;
	BBitmap* mScreenBitmap;
}; 

BScreenSaver *instantiate_screen_saver( BMessage *msg, image_id id ) 
{ 
    return new MinimalSaver( msg, id ); 
} 

MinimalSaver::MinimalSaver( BMessage *archive, image_id image ) 
    : BScreenSaver( archive, image ) 
{     
    SetTickSize(10000);
} 

MinimalSaver::~MinimalSaver( void ) 
{ 
    // Nothing else to do here... 
} 

void MinimalSaver::StartConfig( BView *view ) 
{ 
    view->AddChild( new BStringView( BRect( 10, 10, 200, 35 ), 
                                     B_EMPTY_STRING, 
                                     "zoom screensaver, by d.i.r." ) ); 
} 

status_t MinimalSaver::StartSaver( BView *view, bool preview ) 
{ 
	printf("StartSaver()\n");
	
    if (preview)
    	doingPreview = true;
    else
    	doingPreview = false;

	srandom(system_time());
	
	BScreen screen(B_MAIN_SCREEN_ID);
	screenFrame = screen.Frame();
	screenFrame.PrintToStream();
	
	/* a 32-bit screen bitmap is obviously large and perhaps
	** wasteful when the actual screen bit depth is lower, but
	** it makes grabbing the pixel values in GetPixel easy
	*/
	//we'll save the original screen in here
	mScreenBitmap = new BBitmap(screenFrame,B_RGB32,false);
	
	if (mScreenBitmap->IsValid()) {
		// perhaps BScreen::GetBitmap is the ticket now? Investigate...
		//_get_screen_bitmap_(mScreenBitmap,screenFrame,false);
		screen.ReadBitmap(mScreenBitmap);
	}
	else {
		return (B_ERROR);
	}

    return B_OK; 
} 

void MinimalSaver::Draw( BView *view, int32 frame ) 
{ 
	BRect viewFrame = view->Bounds();
	static BRect sourceRect(-1.0,-1.0,-1.0,-1.0);
	static BRect destRect = sourceRect;
	static int32 previewCount = 0;
	static int xIncrement = 0;
	static int yIncrement = 0;
	static int xMax = 0;
	static int yMax = 0;
	static int xJiggle = 0;
	static int yJiggle = 0;
	static bool newChunk = true;
	
	//if we're in the preview, only draw every 8 frames...
	if (!doingPreview || previewCount % 8 == 0) {
		//pick a new chunk of screen to zoom on...
		if (newChunk) {
			//get new sides
			sourceRect.left = random()%(int32)viewFrame.Width();
			sourceRect.right = sourceRect.left + random()%20 + 10;	
			sourceRect.top = random()%(int32)viewFrame.Height();
			sourceRect.bottom = sourceRect.top + random()%20 + 10;
			//see if we got a valid rect. if not, try again until we do
			while(!viewFrame.Contains(sourceRect)) {
				//get new sides
				sourceRect.left = random()%(int32)viewFrame.Width();
				sourceRect.right = sourceRect.left + random()%20 + 10;	
				sourceRect.top = random()%(int32)viewFrame.Height();
				sourceRect.bottom = sourceRect.top + random()%20 + 10;
			}
			//reset destRect
			destRect = sourceRect;
			//get new x & y increments
			xIncrement = random()%3 + 1;
			yIncrement = random()%3 + 1;
			//get new max size
			xMax = random()%(int32)viewFrame.Width();
			yMax = random()%(int32)viewFrame.Height();
			//get new jiggle factors for moving around
			xJiggle = random()%3 - 1;
			yJiggle = random()%3 - 1;
			
			newChunk = false;
		}
		else {
			//move sourceRect by jiggle factors
			BRect temp = sourceRect;
			temp.OffsetBy(xJiggle,yJiggle);
			//make sure we have a valid rect
			if (viewFrame.Contains(temp))
				sourceRect = temp;
			//if not, pick some new jiggle factors so we can continue moving...
			else {
				//get new jiggle factors for moving around
				xJiggle = random()%3 - 1;
				yJiggle = random()%3 - 1;
			}
			destRect.InsetBy(-xIncrement,-yIncrement);
		}
		
		if (view->Window()->Lock()) {
			//blow up sourceRect to fit into destRect
			view->DrawBitmap(mScreenBitmap, sourceRect, destRect);
			//view->DrawBitmap(mScreenBitmap, sourceRect, sourceRect);
			//stroke outline
			view->SetHighColor(0,0,255);
			view->StrokeRect(destRect);
			//view->StrokeRect(sourceRect);
			view->Window()->Unlock();
		}
	}
	previewCount++;
	//zoom to max width/height, then start again
	if (destRect.Width() >= xMax || destRect.Height() >= yMax)
		newChunk = true;
}